#include "global.hh"
#include "Cfile.hh"

Cfile::Cfile() : m_fp(0)
{
  /* nothing */
}

Cfile::~Cfile()
{
  fclose(m_fp);
}

int Cfile::fopen_rb(std::string binary_filename)
{
  if((m_fp = fopen(binary_filename.c_str(), "rb")) == 0){
    printf("cannot open %s\n", binary_filename.c_str());
    return ERROR;
  }
  return OK;
}

int Cfile::fopen_r(std::string txt_filename)
{
  if((m_fp = fopen(txt_filename.c_str(), "r")) == 0){
    printf("cannot open %s\n", txt_filename.c_str());
    return ERROR;
  }
  return OK;
}

int Cfile::fopen_wb(std::string binary_filename)
{
  if((m_fp = fopen(binary_filename.c_str(), "r")) != 0){
    printf("all ready exist %s\n", binary_filename.c_str());
    return ERROR;
  }else{
    if((m_fp = fopen(binary_filename.c_str(), "wb")) == 0){
      printf("cannot open $s\n", binary_filename.c_str());
      return ERROR;
    }
  }
  return OK;
}

int Cfile::fopen_w(std::string txt_filename)
{
  if((m_fp = fopen(txt_filename.c_str(), "r")) != 0){
    printf("all ready exist %s\n", txt_filename.c_str());
    return ERROR;
  }else{
    if((m_fp = fopen(txt_filename.c_str(), "w")) == 0){
      printf("cannot open $s\n", txt_filename.c_str());
      return ERROR;
    }
  }
  return OK;
}

int Cfile::read_one_block(data_t data_block[])
{
  fread(data_block, sizeof(data_t), BLOCK_SIZE/sizeof(data_t), m_fp);
  if(feof(m_fp)){
    puts("end of file");
    return END_OF_FILE;
  }/*end of file*/
  return OK;
}

int Cfile::read_line(char* buf, size_t size)
{
  if(fgets(buf, size, m_fp) == 0){
    if(feof(m_fp) == 0){
      printf("fgets error");
      exit(EXIT_FAILURE);
    }else{
      return END_OF_FILE;
    }
  }
  return OK;   
}

int Cfile::write_one_block(data_t data_block[])
{
  char buf[6];
  /* unsigned short ならいいのだが signed short だと勝手にintに拡張されてしまう? */
  /* unsigned だと先頭の符号ビットがある時にintに拡張しても正だからなにも起きない */
  /* だけど、signed だと先頭の符号ビットがあるとintに拡張したとき負として読まれる */
  /* そして、上の桁もffffになって、%04xはmaxじゃなくて下限なのでffff8015とかになる */

  for(int i = 0; i < BLOCK_SIZE/sizeof(data_t)/0x8; ++i){
    for(int j = 0; j < 0x8; ++j){
      sprintf(buf, "%04x ", (unsigned short)data_block[i*0x8+j]);
      write_word(buf);
    }
    write_word('\n');
  }
  return OK;
}

int Cfile::write_binary(const void* buf, size_t size)
{
  fwrite((const char*)buf, sizeof(char), size, m_fp);
  return OK;
}

int Cfile::write_word(char buf)
{
  fprintf(m_fp, "%c", buf);
  return OK;
}

int Cfile::write_word(char* buf)
{
  fprintf(m_fp, "%s", buf);
  return OK;
}

int Cfile::write_word(std::string buf)
{
  fprintf(m_fp, "%s", buf.c_str());
  return OK;
}

int Cfile::write_word(int buf)
{
  fprintf(m_fp, "%d", buf);
  return OK;
 }

int Cfile::seek_back_one_block()
{
  fseek(m_fp, -BLOCK_SIZE, SEEK_CUR);
  return OK;
}
